@@ -7,10 +7,23 @@
  * and rip_callback, which is a how you find out whats going on with the rip.
  * the rest of this file is really just boring command line parsing code.
  * and a signal handler for when the user hits CTRL+C
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-
 #if WIN32
-#define sleep	Sleep
+//#define sleep	Sleep
 #include <windows.h>
 #else
 #include <unistd.h>
@@ -24,31 +37,33 @@
 #include <time.h>
 #include "srtypes.h"
 #include "rip_manager.h"
+#include "prefs.h"
 #include "mchar.h"
 #include "filelib.h"
 #include "debug.h"
 
-/*******************************************************************************
+/*****************************************************************************
  * Private functions
- ******************************************************************************/
+ *****************************************************************************/
 static void print_usage();
-static void print_status();
-static void catch_sig(int code);
-static void parse_arguments(int argc, char **argv);
-static void rip_callback(int message, void *data);
-static void parse_extended_options (char* rule);
-static void verify_splitpoint_rules (void);
+static void print_status (RIP_MANAGER_INFO *rmi);
+static void catch_sig (int code);
+static void parse_arguments (STREAM_PREFS *prefs, int argc, char **argv);
+static void rip_callback (RIP_MANAGER_INFO* rmi, int message, void *data);
+static void parse_extended_options (STREAM_PREFS *prefs, char *rule);
+static void verify_splitpoint_rules (STREAM_PREFS *prefs);
+static void print_to_console (char* fmt, ...);
 
-/*******************************************************************************
- * Private Vars 
- ******************************************************************************/
+/*****************************************************************************
+ * Private variables
+ *****************************************************************************/
 static char m_buffer_chars[] = {'\\', '|', '/', '-', '*'}; /* for formating */
-static RIP_MANAGER_INFO 	m_curinfo; /* from the rip_manager callback */
+//static RIP_MANAGER_INFO 	m_curinfo; /* from the rip_manager callback */
 static BOOL			m_started = FALSE;
 static BOOL			m_alldone = FALSE;
 static BOOL			m_got_sig = FALSE;
-static BOOL 			m_dont_print = FALSE;
-RIP_MANAGER_OPTIONS 		m_opt;
+static BOOL			m_dont_print = FALSE;
+static BOOL			m_print_stderr = FALSE;
 time_t				m_stop_time = 0;
 
 /* main()
@@ -63,18 +78,26 @@ time_t				m_stop_time = 0;
  */
 
 int
-main (int argc, char* argv[])
+main (int argc, char *argv[])
 {
     int ret;
     time_t temp_time;
-    signal(SIGINT, catch_sig);
-    signal(SIGTERM, catch_sig);
+    STREAM_PREFS prefs;
+    RIP_MANAGER_INFO *rmi = 0;
 
-    parse_arguments(argc, argv);
-    if (!m_dont_print)
-	fprintf(stderr, "Connecting...\n");
-    if ((ret = rip_manager_start(rip_callback, &m_opt)) != SR_SUCCESS) {
-	fprintf(stderr, "Couldn't connect to %s\n", m_opt.url);
+    signal (SIGINT, catch_sig);
+    signal (SIGTERM, catch_sig);
+
+    parse_arguments (&prefs, argc, argv);
+
+    print_to_console ("Connecting...\n");
+
+    
+    rip_manager_init ();
+
+    /* Launch the ripping thread */
+    if ((ret = rip_manager_start (&rmi, &prefs, rip_callback)) != SR_SUCCESS) {
+	fprintf(stderr, "Couldn't connect to %s\n", prefs.url);
 	exit(1);
     }
 
@@ -83,39 +106,54 @@ main (int argc, char* argv[])
      * (i.e. rip_manager_stop) from a signal handler.. or at least not
      * in FreeBSD 3.4, i don't know about linux or NT.
      */
-    while(!m_got_sig && !m_alldone) {
+    while (!m_got_sig && !m_alldone) {
 	sleep(1);
 	time(&temp_time);
 	if (m_stop_time && (temp_time >= m_stop_time)) {
-	    if (!m_dont_print) {
-		fprintf(stderr, "\n");
-		fprintf(stderr, "Time to stop is here, bailing\n");
-	    }
+	    print_to_console ("\nTime to stop is here, bailing\n");
 	    break; 
 	}	
     }
 
-    if (!m_dont_print) {
-	fprintf(stderr, "shutting down\n");
-    }
-    /* GCS: Why? */
-#if defined (commentout)
-    m_dont_print = TRUE;
-#endif
-    rip_manager_stop();
+    print_to_console ("shutting down\n");
+
+    rip_manager_stop (rmi);
+    rip_manager_cleanup ();
+
     return 0;
 }
 
 void
 catch_sig(int code)
 {
-    if (!m_dont_print)
-	fprintf(stderr, "\n");
+    print_to_console ("\n");
     if (!m_started)
         exit(2);
     m_got_sig = TRUE;
 }
 
+/* In 1.63 I have changed two things: sending output to stdout 
+   instead of stderr, and adding fflush as per the sticky in the forum. */
+static void
+print_to_console (char* fmt, ...)
+{
+    va_list argptr;
+
+    if (!m_dont_print) {
+	va_start (argptr, fmt);
+
+	if (m_print_stderr) {
+	    vfprintf (stderr, fmt, argptr);
+	    fflush (stderr);
+	} else {
+	    vfprintf (stdout, fmt, argptr);
+	    fflush (stdout);
+	}
+
+	va_end (argptr);
+    }
+}
+
 /* 
  * This is to handle RM_UPDATE messages from rip_callback(), and more
  * importantly the RIP_MANAGER_INFO struct. Most of the code here
@@ -123,8 +161,9 @@ catch_sig(int code)
  * much smaller.
  */
 void
-print_status()
+print_status (RIP_MANAGER_INFO *rmi)
 {
+    STREAM_PREFS *prefs = rmi->prefs;
     char status_str[128];
     char filesize_str[64];
     static int buffering_tick = 0;
@@ -133,9 +172,9 @@ print_status()
     if (m_dont_print)
 	return;
 
-    if (printed_fullinfo && m_curinfo.filename[0]) {
+    if (printed_fullinfo && rmi->filename[0]) {
 
-	switch(m_curinfo.status)
+	switch(rmi->status)
 	{
 	case RM_STATUS_BUFFERING:
 	    buffering_tick++;
@@ -145,47 +184,46 @@ print_status()
 	    sprintf(status_str,"buffering - %c ",
 		    m_buffer_chars[buffering_tick]);
 
-	    fprintf(stderr, "[%14s] %.50s\r",
-		    status_str,
-		    m_curinfo.filename);
+	    print_to_console ("[%14s] %.50s\r",
+			      status_str,
+			      rmi->filename);
 	    break;
 
 	case RM_STATUS_RIPPING:
-	    if (m_curinfo.track_count < m_opt.dropcount) {
+	    if (rmi->track_count < prefs->dropcount) {
 		strcpy(status_str, "skipping...   ");
 	    } else {
 		strcpy(status_str, "ripping...    ");
 	    }
-	    format_byte_size(filesize_str, m_curinfo.filesize);
-	    fprintf(stderr, "[%14s] %.50s [%7s]\r",
-		    status_str,
-		    m_curinfo.filename,
-		    filesize_str);
+	    format_byte_size(filesize_str, rmi->filesize);
+	    print_to_console ("[%14s] %.50s [%7s]\r",
+			      status_str,
+			      rmi->filename,
+			      filesize_str);
 	    break;
 	case RM_STATUS_RECONNECTING:
 	    strcpy(status_str, "re-connecting..");
-	    fprintf(stderr, "[%14s]\r", status_str);
+	    print_to_console ("[%14s]\r", status_str);
 	    break;
 	}
 			
     }
     if (!printed_fullinfo)
     {
-	fprintf(stderr, 
-		"stream: %s\n"
-		"server name: %s\n"
-		"bitrate: %d\n"
-		"meta interval: %d\n",
-		m_curinfo.streamname,
-		m_curinfo.server_name,
-		m_curinfo.bitrate,
-		m_curinfo.meta_interval);
-	if(GET_MAKE_RELAY(m_opt.flags))
+	print_to_console ("stream: %s\n"
+			  "server name: %s\n"
+			  "bitrate: %d\n"
+			  "meta interval: %d\n",
+			  rmi->streamname,
+			  rmi->server_name,
+			  rmi->bitrate,
+			  rmi->meta_interval);
+	if(GET_MAKE_RELAY(prefs->flags))
 	{
-	    fprintf(stderr, "relay port: %d\n"
-		    "[%14s]\r",
-		    m_opt.relay_port,
-		    "getting track name... ");
+	    print_to_console ("relay port: %d\n"
+			      "[%14s]\r",
+			      prefs->relay_port,
+			      "getting track name... ");
 	}
 
 	printed_fullinfo = TRUE;
@@ -201,16 +239,13 @@ print_status()
  * and prints out stuff to the screen.
  */
 void
-rip_callback(int message, void *data)
+rip_callback (RIP_MANAGER_INFO* rmi, int message, void *data)
 {
-    RIP_MANAGER_INFO *info;
     ERROR_INFO *err;
     switch(message)
     {
     case RM_UPDATE:
-	info = (RIP_MANAGER_INFO*)data;
-	memcpy(&m_curinfo, info, sizeof(RIP_MANAGER_INFO));
-	print_status();
+	print_status (rmi);
 	break;
     case RM_ERROR:
 	err = (ERROR_INFO*)data;
@@ -218,13 +253,11 @@ rip_callback(int message, void *data)
 	m_alldone = TRUE;
 	break;
     case RM_DONE:
-	if (!m_dont_print)
-	    fprintf(stderr, "bye..\n");
+	print_to_console ("bye..\n");
 	m_alldone = TRUE;
 	break;
     case RM_NEW_TRACK:
-	if (!m_dont_print)
-	    fprintf(stderr, "\n");
+	print_to_console ("\n");
 	break;
     case RM_STARTED:
 	m_started = TRUE;
@@ -232,7 +265,7 @@ rip_callback(int message, void *data)
     }
 }
 
-void
+static void
 print_usage()
 {
     fprintf(stderr, "Usage: streamripper URL [OPTIONS]\n");
@@ -257,19 +290,21 @@ print_usage()
     fprintf(stderr, "      -u useragent   - Use a different UserAgent than \"Streamripper\"\n");
     fprintf(stderr, "      -w rulefile    - Parse metadata using rules in file.\n");
     fprintf(stderr, "      -m timeout     - Number of seconds before force-closing stalled conn\n");
-    fprintf(stderr, "      -k count       - Skip over first <count> tracks before starting to rip\n");
+    fprintf(stderr, "      -k count       - Leave <count> tracks in incomplete\n");
 #if !defined (WIN32)
     fprintf(stderr, "      -I interface   - Rip from specified interface (e.g. eth0)\n");
 #endif
     fprintf(stderr, "      -T             - Truncate duplicated tracks in incomplete\n");
     fprintf(stderr, "      -E command     - Run external command to fetch metadata\n");
     fprintf(stderr, "      --quiet        - Don't print ripping status to console\n");
+    fprintf(stderr, "      --stderr       - Print ripping status to stderr (old behavior)\n");
     fprintf(stderr, "      --debug        - Save debugging trace\n");
     fprintf(stderr, "ID3 opts (mp3/aac/nsv):  [The default behavior is adding ID3V2.3 only]\n");
     fprintf(stderr, "      -i                           - Don't add any ID3 tags to output file\n");
     fprintf(stderr, "      --with-id3v1                 - Add ID3V1 tags to output file\n");
     fprintf(stderr, "      --without-id3v2              - Don't add ID3V2 tags to output file\n");
     fprintf(stderr, "Splitpoint opts (mp3 only):\n");
+    fprintf(stderr, "      --xs2                        - Use new algorithm for silence detection\n");
     fprintf(stderr, "      --xs-offset=num              - Shift relative to metadata (msec)\n");
     fprintf(stderr, "      --xs-padding=num:num         - Add extra to prev:next track (msec)\n");
     fprintf(stderr, "      --xs-search-window=num:num   - Search window relative to metadata (msec)\n");
@@ -287,8 +322,8 @@ print_usage()
  * port of it under Win32.. there probably is one, maybe i didn't look 
  * hard enough. 
  */
-void
-parse_arguments(int argc, char **argv)
+static void
+parse_arguments (STREAM_PREFS* prefs, int argc, char **argv)
 {
     int i;
     char *c;
@@ -298,18 +333,20 @@ parse_arguments(int argc, char **argv)
 	exit(2);
     }
 
-    // Set default options
-    set_rip_manager_options_defaults (&m_opt);
-
     // Get URL
-    strncpy (m_opt.url, argv[1], MAX_URL_LEN);
+    strncpy (prefs->url, argv[1], MAX_URL_LEN);
+
+    // Load prefs
+    prefs_load ();
+    prefs_get_stream_prefs (prefs, prefs->url);
+    prefs_save ();
 
     // Parse arguments
-    for(i = 1; i < argc; i++) {
+    for (i = 1; i < argc; i++) {
 	if (argv[i][0] != '-')
 	    continue;
 
-	c = strchr("dDEfIklLmMopRuw", argv[i][1]);
+	c = strchr ("dDEfIklLmMopRuw", argv[i][1]);
         if (c != NULL) {
             if ((i == (argc-1)) || (argv[i+1][0] == '-')) {
 		fprintf(stderr, "option %s requires an argument\n", argv[i]);
@@ -320,31 +357,31 @@ parse_arguments(int argc, char **argv)
 	{
 	case 'a':
 	    /* Create single file output + cue sheet */
-	    m_opt.flags |= OPT_SINGLE_FILE_OUTPUT;
-	    m_opt.showfile_pattern[0] = 0;
+	    prefs->flags |= OPT_SINGLE_FILE_OUTPUT;
+	    prefs->showfile_pattern[0] = 0;
 	    if (i == (argc-1) || argv[i+1][0] == '-')
 		break;
 	    i++;
-	    strncpy (m_opt.showfile_pattern, argv[i], SR_MAX_PATH);
+	    strncpy (prefs->showfile_pattern, argv[i], SR_MAX_PATH);
 	    break;
 	case 'A':
-	    m_opt.flags ^= OPT_INDIVIDUAL_TRACKS;
+	    prefs->flags |= OPT_INDIVIDUAL_TRACKS;
 	    break;
 	case 'c':
-	    m_opt.flags ^= OPT_AUTO_RECONNECT;
+	    prefs->flags |= OPT_AUTO_RECONNECT;
 	    break;
 	case 'd':
 	    i++;
-	    strncpy(m_opt.output_directory, argv[i], SR_MAX_PATH);
+	    strncpy(prefs->output_directory, argv[i], SR_MAX_PATH);
 	    break;
 	case 'D':
 	    i++;
-	    strncpy(m_opt.output_pattern, argv[i], SR_MAX_PATH);
+	    strncpy(prefs->output_pattern, argv[i], SR_MAX_PATH);
 	    break;
 	case 'E':
-	    m_opt.flags |= OPT_EXTERNAL_CMD;
+	    prefs->flags |= OPT_EXTERNAL_CMD;
 	    i++;
-	    strncpy(m_opt.ext_cmd, argv[i], SR_MAX_PATH);
+	    strncpy(prefs->ext_cmd, argv[i], SR_MAX_PATH);
 	    break;
 	case 'f':
 	    i++;
@@ -356,17 +393,16 @@ parse_arguments(int argc, char **argv)
             exit(0);
 	    break;
 	case 'i':
-	    //	    m_opt.flags ^= OPT_ADD_ID3;
-	    OPT_FLAG_SET(m_opt.flags,OPT_ADD_ID3V1,0);
-	    OPT_FLAG_SET(m_opt.flags,OPT_ADD_ID3V2,0);
+	    OPT_FLAG_SET(prefs->flags,OPT_ADD_ID3V1,0);
+	    OPT_FLAG_SET(prefs->flags,OPT_ADD_ID3V2,0);
 	    break;
 	case 'I':
 	    i++;
-	    strncpy(m_opt.if_name, argv[i], SR_MAX_PATH);
+	    strncpy(prefs->if_name, argv[i], SR_MAX_PATH);
 	    break;
 	case 'k':
 	    i++;
-	    m_opt.dropcount = atoi(argv[i]);
+	    prefs->dropcount = atoi(argv[i]);
 	    break;
 	case 'l':
 	    i++;
@@ -375,21 +411,21 @@ parse_arguments(int argc, char **argv)
 	    break;
 	case 'L':
 	    i++;
-	    strncpy(m_opt.pls_file, argv[i], SR_MAX_PATH);
+	    strncpy(prefs->pls_file, argv[i], SR_MAX_PATH);
 	    break;
 	case 'm':
 	    i++;
-	    m_opt.timeout = atoi(argv[i]);
+	    prefs->timeout = atoi(argv[i]);
 	    break;
  	case 'M':
  	    i++;
- 	    m_opt.maxMB_rip_size = atoi(argv[i]);
- 	    m_opt.flags |= OPT_CHECK_MAX_BYTES;
+ 	    prefs->maxMB_rip_size = atoi(argv[i]);
+ 	    prefs->flags |= OPT_CHECK_MAX_BYTES;
  	    break;
 	case 'o':
 	    i++;
-	    m_opt.overwrite = string_to_overwrite_opt (argv[i]);
-	    if (m_opt.overwrite == OVERWRITE_UNKNOWN) {
+	    prefs->overwrite = string_to_overwrite_opt (argv[i]);
+	    if (prefs->overwrite == OVERWRITE_UNKNOWN) {
 		printf ("Error: -o option requires an argument\n"
 			"Please use \"-o always\" or \"-o never\"\n");
 		exit (1);
@@ -397,7 +433,7 @@ parse_arguments(int argc, char **argv)
 	    break;
 	case 'p':
 	    i++;
-	    strncpy(m_opt.proxyurl, argv[i], MAX_URL_LEN);
+	    strncpy(prefs->proxyurl, argv[i], MAX_URL_LEN);
 	    break;
 	case 'P':
 	    i++;
@@ -405,63 +441,63 @@ parse_arguments(int argc, char **argv)
 		    "Please use -D pattern instead.\n");
 	    exit (1);
 	case 'q':
-	    m_opt.flags ^= OPT_COUNT_FILES;
-	    m_opt.count_start = -1;     /* -1 means auto-detect */
+	    prefs->flags |= OPT_COUNT_FILES;
+	    prefs->count_start = -1;     /* -1 means auto-detect */
 	    if (i == (argc-1) || argv[i+1][0] == '-')
 		break;
 	    i++;
-	    m_opt.count_start = atoi(argv[i]);
+	    prefs->count_start = atoi(argv[i]);
 	    break;
 	case 'r':
-	    m_opt.flags ^= OPT_MAKE_RELAY;
+	    prefs->flags |= OPT_MAKE_RELAY;
 	    if (i == (argc-1) || argv[i+1][0] == '-')
 		break;
 	    i++;
 	    c = strstr(argv[i], ":");
 	    if (NULL == c) {
-	    	m_opt.relay_port = atoi(argv[i]);
+	    	prefs->relay_port = atoi(argv[i]);
 	    } else {
 	    	*c = '\0';
-		strncpy(m_opt.relay_ip, argv[i], SR_MAX_PATH);
-		m_opt.relay_port = atoi(++c);
+		strncpy(prefs->relay_ip, argv[i], SR_MAX_PATH);
+		prefs->relay_port = atoi(++c);
  	    }
 	    break;
 	case 'R':
 	    i++;
-	    m_opt.max_connections = atoi(argv[i]);
+	    prefs->max_connections = atoi(argv[i]);
 	    break;
 	case 's':
-	    m_opt.flags ^= OPT_SEPERATE_DIRS;
+	    prefs->flags |= OPT_SEPERATE_DIRS;
 	    break;
 	case 't':
-	    m_opt.flags |= OPT_KEEP_INCOMPLETE;
+	    prefs->flags |= OPT_KEEP_INCOMPLETE;
 	    break;
 	case 'T':
-	    m_opt.flags |= OPT_TRUNCATE_DUPS;
+	    prefs->flags |= OPT_TRUNCATE_DUPS;
 	    break;
 	case 'u':
 	    i++;
-	    strncpy(m_opt.useragent, argv[i], MAX_USERAGENT_STR);
+	    strncpy(prefs->useragent, argv[i], MAX_USERAGENT_STR);
 	    break;
 	case 'v':
 	    printf("Streamripper %s\n", SRVERSION);
 	    exit(0);
 	case 'w':
 	    i++;
-	    strncpy(m_opt.rules_file, argv[i], SR_MAX_PATH);
+	    strncpy(prefs->rules_file, argv[i], SR_MAX_PATH);
 	    break;
 	case 'z':
-	    m_opt.flags ^= OPT_SEARCH_PORTS;
-	    m_opt.max_port = m_opt.relay_port+1000;
+	    prefs->flags |= OPT_SEARCH_PORTS;
+	    prefs->max_port = prefs->relay_port+1000;
 	    break;
 	case '-':
-	    parse_extended_options(&argv[i][2]);
+	    parse_extended_options (prefs, &argv[i][2]);
 	    break;
 	}
     }
 
     /* Need to verify that splitpoint rules were sane */
-    verify_splitpoint_rules ();
+    verify_splitpoint_rules (prefs);
 
     /* Verify that first parameter is URL */
     if (argv[1][0] == '-') {
@@ -471,7 +507,7 @@ parse_arguments(int argc, char **argv)
 }
 
 static void
-parse_extended_options (char* rule)
+parse_extended_options (STREAM_PREFS* prefs, char* rule)
 {
     int x,y;
 
@@ -486,6 +522,10 @@ parse_extended_options (char* rule)
 	debug_enable();
 	return;
     }
+    if (!strcmp(rule,"stderr")) {
+	m_print_stderr = TRUE;
+	return;
+    }
     if (!strcmp(rule,"quiet")) {
 	m_dont_print = TRUE;
 	return;
@@ -494,88 +534,93 @@ parse_extended_options (char* rule)
     /* Splitpoint options */
     if ((!strcmp(rule,"xs-none"))
 	|| (!strcmp(rule,"xs_none"))) {
-	m_opt.sp_opt.xs = 0;
+	prefs->sp_opt.xs = 0;
 	debug_printf ("Disable silence detection");
 	return;
     }
+    if (!strcmp(rule,"xs2")) {
+	prefs->sp_opt.xs = 2;
+	debug_printf ("Setting xs2\n");
+	return;
+    }
     if ((1==sscanf(rule,"xs-min-volume=%d",&x)) 
 	|| (1==sscanf(rule,"xs_min_volume=%d",&x))) {
-	m_opt.sp_opt.xs_min_volume = x;
+	prefs->sp_opt.xs_min_volume = x;
 	debug_printf ("Setting minimum volume to %d\n",x);
 	return;
     }
     if ((1==sscanf(rule,"xs-silence-length=%d",&x))
 	|| (1==sscanf(rule,"xs_silence_length=%d",&x))) {
-	m_opt.sp_opt.xs_silence_length = x;
+	prefs->sp_opt.xs_silence_length = x;
 	debug_printf ("Setting silence length to %d\n",x);
 	return;
     }
     if ((2==sscanf(rule,"xs-search-window=%d:%d",&x,&y))
 	|| (2==sscanf(rule,"xs_search_window=%d:%d",&x,&y))) {
-	m_opt.sp_opt.xs_search_window_1 = x;
-	m_opt.sp_opt.xs_search_window_2 = y;
+	prefs->sp_opt.xs_search_window_1 = x;
+	prefs->sp_opt.xs_search_window_2 = y;
 	debug_printf ("Setting search window to (%d:%d)\n",x,y);
 	return;
     }
     if ((1==sscanf(rule,"xs-offset=%d",&x))
 	|| (1==sscanf(rule,"xs_offset=%d",&x))) {
-	m_opt.sp_opt.xs_offset = x;
+	prefs->sp_opt.xs_offset = x;
 	debug_printf ("Setting silence offset to %d\n",x);
 	return;
     }
     if ((2==sscanf(rule,"xs-padding=%d:%d",&x,&y))
 	|| (2==sscanf(rule,"xs_padding=%d:%d",&x,&y))) {
-	m_opt.sp_opt.xs_padding_1 = x;
-	m_opt.sp_opt.xs_padding_2 = y;
+	prefs->sp_opt.xs_padding_1 = x;
+	prefs->sp_opt.xs_padding_2 = y;
 	debug_printf ("Setting file output padding to (%d:%d)\n",x,y);
 	return;
     }
 
     /* id3 options */
     if (!strcmp(rule,"with-id3v2")) {
-	OPT_FLAG_SET(m_opt.flags,OPT_ADD_ID3V2,1);
+	OPT_FLAG_SET(prefs->flags,OPT_ADD_ID3V2,1);
 	return;
     }
     if (!strcmp(rule,"without-id3v2")) {
-	OPT_FLAG_SET(m_opt.flags,OPT_ADD_ID3V2,0);
+	OPT_FLAG_SET(prefs->flags,OPT_ADD_ID3V2,0);
 	return;
     }
     if (!strcmp(rule,"with-id3v1")) {
-	OPT_FLAG_SET(m_opt.flags,OPT_ADD_ID3V1,1);
+	OPT_FLAG_SET(prefs->flags,OPT_ADD_ID3V1,1);
 	return;
     }
     if (!strcmp(rule,"without-id3v1")) {
-	OPT_FLAG_SET(m_opt.flags,OPT_ADD_ID3V1,0);
+	OPT_FLAG_SET(prefs->flags,OPT_ADD_ID3V1,0);
 	return;
     }
 
     /* codeset options */
     x = strlen("codeset-filesys=");
     if (!strncmp(rule,"codeset-filesys=",x)) {
-	strncpy (m_opt.cs_opt.codeset_filesys, &rule[x], MAX_CODESET_STRING);
+	strncpy (prefs->cs_opt.codeset_filesys, &rule[x], MAX_CODESET_STRING);
 	debug_printf ("Setting filesys codeset to %s\n",
-		      m_opt.cs_opt.codeset_filesys);
+		      prefs->cs_opt.codeset_filesys);
 	return;
     }
     x = strlen("codeset-id3=");
     if (!strncmp(rule,"codeset-id3=",x)) {
-	strncpy (m_opt.cs_opt.codeset_id3, &rule[x], MAX_CODESET_STRING);
+	strncpy (prefs->cs_opt.codeset_id3, &rule[x], MAX_CODESET_STRING);
 	debug_printf ("Setting id3 codeset to %s\n",
-		      m_opt.cs_opt.codeset_id3);
+		      prefs->cs_opt.codeset_id3);
 	return;
     }
     x = strlen("codeset-metadata=");
     if (!strncmp(rule,"codeset-metadata=",x)) {
-	strncpy (m_opt.cs_opt.codeset_metadata, &rule[x], MAX_CODESET_STRING);
+	strncpy (prefs->cs_opt.codeset_metadata, &rule[x], MAX_CODESET_STRING);
 	debug_printf ("Setting metadata codeset to %s\n",
-		      m_opt.cs_opt.codeset_metadata);
+		      prefs->cs_opt.codeset_metadata);
 	return;
     }
     x = strlen("codeset-relay=");
     if (!strncmp(rule,"codeset-relay=",x)) {
-	strncpy (m_opt.cs_opt.codeset_relay, &rule[x], MAX_CODESET_STRING);
+	strncpy (prefs->cs_opt.codeset_relay, &rule[x], MAX_CODESET_STRING);
 	debug_printf ("Setting relay codeset to %s\n",
-		      m_opt.cs_opt.codeset_relay);
+		      prefs->cs_opt.codeset_relay);
 	return;
     }
 
@@ -585,7 +630,7 @@ parse_extended_options (char* rule)
 }
 
 static void
-verify_splitpoint_rules (void)
+verify_splitpoint_rules (STREAM_PREFS *prefs)
 {
 #if defined (commentout)
     /* This is still not complete, but the warning causes people to 
@@ -594,41 +639,41 @@ verify_splitpoint_rules (void)
 #endif
     
     /* xs_silence_length must be non-negative and divisible by two */
-    if (m_opt.sp_opt.xs_silence_length < 0) {
-	m_opt.sp_opt.xs_silence_length = 0;
+    if (prefs->sp_opt.xs_silence_length < 0) {
+	prefs->sp_opt.xs_silence_length = 0;
     }
-    if (m_opt.sp_opt.xs_silence_length % 2) {
-        m_opt.sp_opt.xs_silence_length ++;
+    if (prefs->sp_opt.xs_silence_length % 2) {
+        prefs->sp_opt.xs_silence_length ++;
     }
 
     /* search_window values must be non-negative */
-    if (m_opt.sp_opt.xs_search_window_1 < 0) {
-	m_opt.sp_opt.xs_search_window_1 = 0;
+    if (prefs->sp_opt.xs_search_window_1 < 0) {
+	prefs->sp_opt.xs_search_window_1 = 0;
     }
-    if (m_opt.sp_opt.xs_search_window_2 < 0) {
-	m_opt.sp_opt.xs_search_window_2 = 0;
+    if (prefs->sp_opt.xs_search_window_2 < 0) {
+	prefs->sp_opt.xs_search_window_2 = 0;
     }
 
     /* if silence_length is 0, then search window should be zero */
-    if (m_opt.sp_opt.xs_silence_length == 0) {
-	m_opt.sp_opt.xs_search_window_1 = 0;
-	m_opt.sp_opt.xs_search_window_2 = 0;
+    if (prefs->sp_opt.xs_silence_length == 0) {
+	prefs->sp_opt.xs_search_window_1 = 0;
+	prefs->sp_opt.xs_search_window_2 = 0;
     }
 
     /* search_window values must be longer than silence_length */
-    if (m_opt.sp_opt.xs_search_window_1 + m_opt.sp_opt.xs_search_window_2
-	    < m_opt.sp_opt.xs_silence_length) {
+    if (prefs->sp_opt.xs_search_window_1 + prefs->sp_opt.xs_search_window_2
+	    < prefs->sp_opt.xs_silence_length) {
 	/* if this happens, disable search */
-	m_opt.sp_opt.xs_search_window_1 = 0;
-	m_opt.sp_opt.xs_search_window_2 = 0;
-	m_opt.sp_opt.xs_silence_length = 0;
+	prefs->sp_opt.xs_search_window_1 = 0;
+	prefs->sp_opt.xs_search_window_2 = 0;
+	prefs->sp_opt.xs_silence_length = 0;
     }
 
     /* search window lengths must be at least 1/2 of silence_length */
-    if (m_opt.sp_opt.xs_search_window_1 < m_opt.sp_opt.xs_silence_length) {
-	m_opt.sp_opt.xs_search_window_1 = m_opt.sp_opt.xs_silence_length;
+    if (prefs->sp_opt.xs_search_window_1 < prefs->sp_opt.xs_silence_length) {
+	prefs->sp_opt.xs_search_window_1 = prefs->sp_opt.xs_silence_length;
     }
-    if (m_opt.sp_opt.xs_search_window_2 < m_opt.sp_opt.xs_silence_length) {
-	m_opt.sp_opt.xs_search_window_2 = m_opt.sp_opt.xs_silence_length;
+    if (prefs->sp_opt.xs_search_window_2 < prefs->sp_opt.xs_silence_length) {
+	prefs->sp_opt.xs_search_window_2 = prefs->sp_opt.xs_silence_length;
     }
 }
