@@ -18,6 +18,7 @@
 #define __FILELIB_H__
 
 #include "srtypes.h"
+#include "errors.h"
 #if WIN32
 #include <windows.h>
 #endif
@@ -67,7 +68,8 @@
 
 
 error_code
-filelib_init (BOOL do_individual_tracks,
+filelib_init (RIP_MANAGER_INFO* rmi,
+	      BOOL do_individual_tracks,
 	      BOOL do_count,
 	      int count_start,
 	      BOOL keep_incomplete,
@@ -79,13 +81,15 @@ filelib_init (BOOL do_individual_tracks,
 	      int get_separate_dirs,
 	      int get_date_stamp,
 	      char* icy_name);
-error_code filelib_start (TRACK_INFO* ti);
-error_code filelib_end (TRACK_INFO* ti, enum OverwriteOpt overwrite,
-			BOOL truncate_dup, mchar *fullpath);
-error_code filelib_write_track(char *buf, u_long size);
-error_code filelib_write_show(char *buf, u_long size);
-void filelib_shutdown();
-error_code filelib_remove(char *filename);
-error_code filelib_write_cue(TRACK_INFO* ti, int secs);
+error_code filelib_start (RIP_MANAGER_INFO* rmi, TRACK_INFO* ti);
+error_code filelib_write_track (RIP_MANAGER_INFO* rmi, char *buf, u_long size);
+error_code
+filelib_write_show (RIP_MANAGER_INFO* rmi, char *buf, u_long size);
+error_code filelib_write_cue (RIP_MANAGER_INFO* rmi, TRACK_INFO* ti, int secs);
+error_code
+filelib_end (RIP_MANAGER_INFO* rmi,
+	     TRACK_INFO* ti, enum OverwriteOpt overwrite,
+	     BOOL truncate_dup, mchar *fullpath);
+void filelib_shutdown (RIP_MANAGER_INFO* rmi);
 
 #endif //FILELIB
