@@ -15,6 +15,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include "compat.h"
 #include "srtypes.h"
 #include "socklib.h"
@@ -27,9 +28,16 @@
  *****************************************************************************/
 static char* make_auth_header(const char *header_name, 
 			      const char *username, const char *password);
-static error_code httplib_get_sc_header(const char* url, HSOCKET *sock, 
-					SR_HTTP_HEADER *info);
 static char* b64enc(const char *buf, int size);
+static error_code
+http_get_pls (RIP_MANAGER_INFO* rmi, HSOCKET *sock, SR_HTTP_HEADER *info);
+static error_code
+http_get_m3u (RIP_MANAGER_INFO* rmi, HSOCKET *sock, SR_HTTP_HEADER *info);
+static error_code
+http_get_sc_header(RIP_MANAGER_INFO* rmi, const char* url, 
+		   HSOCKET *sock, SR_HTTP_HEADER *info);
+static error_code
+http_parse_url(const char *url, URLINFO *urlinfo);
 
 /******************************************************************************
  * Private Vars
@@ -39,13 +47,13 @@ static char* b64enc(const char *buf, int size);
 
 
 /******************************************************************************
- * Functions
+ * Public functions
  *****************************************************************************/
-
 /* Connect to a shoutcast type stream, leaves when it's about to 
    get the header info */
 error_code
-httplib_sc_connect (HSOCKET *sock, const char *url, const char *proxyurl, 
+http_sc_connect (RIP_MANAGER_INFO* rmi,
+		 HSOCKET *sock, const char *url, const char *proxyurl, 
 		 SR_HTTP_HEADER *info, char *useragent, char *if_name)
 {
     char headbuf[MAX_HEADER_LEN];
@@ -54,41 +62,41 @@ httplib_sc_connect (HSOCKET *sock, const char *url, const char *proxyurl,
 
     while (1) {
 	debug_printf ("***** URL = %s *****\n", url);
-	debug_printf("inet_sc_connect(): calling httplib_parse_url\n");
+	debug_printf("http_sc_connect(): calling http_parse_url\n");
 	if (proxyurl) {
-	    if ((ret = httplib_parse_url(proxyurl, &url_info)) != SR_SUCCESS) {
+	    if ((ret = http_parse_url(proxyurl, &url_info)) != SR_SUCCESS) {
 		return ret;
 	    }
-	} else if ((ret = httplib_parse_url(url, &url_info)) != SR_SUCCESS) {
+	} else if ((ret = http_parse_url(url, &url_info)) != SR_SUCCESS) {
 	    return ret;
 	}
 
-	debug_printf("inet_sc_connect(): calling sockinit\n");
+	debug_printf("http_sc_connect(): calling socklib_init\n");
 	if ((ret = socklib_init()) != SR_SUCCESS)
 	    return ret;
 
-	debug_printf("inet_sc_connect(): calling sock_open: host=%s, port=%d\n",
+	debug_printf("http_sc_connect(): calling socklib_open: host=%s, port=%d\n",
 	    url_info.host, url_info.port);
 	if ((ret = socklib_open(sock, url_info.host, url_info.port, if_name)) != SR_SUCCESS)
 	    return ret;
 
-	debug_printf("inet_sc_connect(): calling httplib_construct_sc_request\n");
-	if ((ret = httplib_construct_sc_request(url, proxyurl, headbuf, useragent)) != SR_SUCCESS)
+	debug_printf("http_sc_connect(): calling http_construct_sc_request\n");
+	if ((ret = http_construct_sc_request(url, proxyurl, headbuf, useragent)) != SR_SUCCESS)
 	    return ret;
 
-	debug_printf("inet_sc_connect(): calling socklib_sendall\n");
+	debug_printf("http_sc_connect(): calling socklib_sendall\n");
 	if ((ret = socklib_sendall(sock, headbuf, strlen(headbuf))) < 0)
 	    return ret;
 
-	debug_printf("inet_sc_connect(): calling get_sc_header\n");
-	if ((ret = httplib_get_sc_header(url, sock, info)) != SR_SUCCESS)
+	debug_printf("http_sc_connect(): calling http_get_sc_header\n");
+	if ((ret = http_get_sc_header(rmi, url, sock, info)) != SR_SUCCESS)
 	    return ret;
 
 	if (*info->http_location) {
 	    /* RECURSIVE CASE */
 	    debug_printf ("Redirecting: %s\n", info->http_location);
 	    url = info->http_location;
-	    //inet_sc_connect(sock, info->http_location, proxyurl, info, useragent, if_name);
+	    //http_sc_connect(sock, info->http_location, proxyurl, info, useragent, if_name);
 	} else {
 	    break;
 	}
@@ -97,34 +105,61 @@ httplib_sc_connect (HSOCKET *sock, const char *url, const char *proxyurl,
     return SR_SUCCESS;
 }
 
+/******************************************************************************
+ * Private functions
+ *****************************************************************************/
+/* See http://www.ietf.org/rfc/rfc3986.txt */
+static void
+unescape_pct_encoding (char* s)
+{
+    char* d = s;
+    char c[3];
+
+    while (*s) {
+	if (*s == '%' && isxdigit(*(s+1)) && isxdigit(*(s+2))) {
+	    c[0] = *(s+1);
+	    c[1] = *(s+2);
+	    c[2] = 0;
+	    *d++ = strtol (c, 0, 16);
+	    s += 3;
+	} else {
+	    *d++ = *s++;
+	}
+    }
+    *d = 0;
+}
+
 /*
  * Parse's a url as in http://host:port/path or host/path, etc..
  * and now http://username:password@server:4480
  */
-error_code
-httplib_parse_url(const char *url, URLINFO *urlinfo)
+static error_code
+http_parse_url(const char *url, URLINFO *urlinfo)
 { 
-    /* see if we have a proto */
-    char *s = strstr(url, "://");
     int ret;
+    char *s;
 
     /* if we have a proto, just skip it. should we care about 
        the proto? like fail if it's not http? */
+    s = strstr(url, "://");
     if (s) url = s + strlen("://");
     memcpy(urlinfo->path, (void *)"/\0", 2);
 
     /* search for a login '@' token */
     if (strchr(url, '@') != NULL) {
 	ret = sscanf(url, "%[^:]:%[^@]", urlinfo->username, urlinfo->password);
-#if defined (commentout)
-	if (ret < 2) return SR_ERROR_PARSE_FAILURE;
-#endif
 	if (ret < 1) {
 	    return SR_ERROR_PARSE_FAILURE;
 	} else if (ret == 1) {
 	    urlinfo->password[0] = '\0';
 	}
 	url = strchr(url, '@') + 1;
+	debug_printf ("Username (escaped): %s\n", urlinfo->username);
+	debug_printf ("Password (escaped): %s\n", urlinfo->password);
+	unescape_pct_encoding (urlinfo->username);
+	unescape_pct_encoding (urlinfo->password);
+	debug_printf ("Username (unescaped): %s\n", urlinfo->username);
+	debug_printf ("Password (unescaped): %s\n", urlinfo->password);
     } else {
 	urlinfo->username[0] = '\0';
 	urlinfo->password[0] = '\0';
@@ -132,11 +167,13 @@ httplib_parse_url(const char *url, URLINFO *urlinfo)
 
     /* search for a port seperator */
     if (strchr(url, ':') != NULL) {
+	debug_printf ("Branch 1 (%s)\n", url);
 	ret = sscanf(url, "%[^:]:%hu/%s", urlinfo->host, 
 		     (short unsigned int*)&urlinfo->port, urlinfo->path+1);
 	if (urlinfo->port < 1) return SR_ERROR_PARSE_FAILURE;
 	ret -= 1;
     } else {
+	debug_printf ("Branch 2 (%s)\n", url);
 	urlinfo->port = 80;
 	ret = sscanf(url, "%[^/]/%s", urlinfo->host, urlinfo->path+1);
     }
@@ -146,18 +183,18 @@ httplib_parse_url(const char *url, URLINFO *urlinfo)
 }
 
 error_code
-httplib_construct_sc_request(const char *url, const char* proxyurl, char *buffer, char *useragent)
+http_construct_sc_request(const char *url, const char* proxyurl, char *buffer, char *useragent)
 {
     int ret;
     URLINFO ui;
     URLINFO proxyui;
     char myurl[MAX_URL_LEN];
-    if ((ret = httplib_parse_url(url, &ui)) != SR_SUCCESS)
+    if ((ret = http_parse_url(url, &ui)) != SR_SUCCESS)
 	return ret;
 
     if (proxyurl) {
 	sprintf(myurl, "http://%s:%d%s", ui.host, ui.port, ui.path);
-	if ((ret = httplib_parse_url(proxyurl, &proxyui)) != SR_SUCCESS)
+	if ((ret = http_parse_url(proxyurl, &proxyui)) != SR_SUCCESS)
 	    return ret;
     } else {
 	strcpy(myurl, ui.path);
@@ -204,6 +241,8 @@ httplib_construct_sc_request(const char *url, const char* proxyurl, char *buffer
     
     strcat(buffer, "\r\n");
 
+    debug_printf ("SC REQUEST:\n%s", buffer);
+
     return SR_SUCCESS;
 }
 
@@ -227,12 +266,12 @@ make_auth_header (const char *header_name, const char *username,
 
 // Here we pretend we're IE 5, hehe
 error_code
-httplib_construct_page_request (const char *url, BOOL proxyformat, char *buffer)
+http_construct_page_request (const char *url, BOOL proxyformat, char *buffer)
 {
     int ret;
     URLINFO ui;
     char myurl[MAX_URL_LEN];
-    if ((ret = httplib_parse_url(url, &ui)) != SR_SUCCESS)
+    if ((ret = http_parse_url(url, &ui)) != SR_SUCCESS)
 	return ret;
 
     if (proxyformat)
@@ -258,11 +297,11 @@ httplib_construct_page_request (const char *url, BOOL proxyformat, char *buffer)
 
 /* Return 1 if a match was found, 0 if not found */
 int
-extract_header_value (char *header, char *dest, char *match)
+extract_header_value (char *header, char *dest, char *match, int maxlen)
 {
     char* start = (char *)strstr(header, match);
     if (start) {
-	subnstr_until(start+strlen(match), "\n", dest, MAX_ICY_STRING);
+	subnstr_until(start+strlen(match), "\n", dest, maxlen);
 	return 1;
     } else {
 	return 0;
@@ -270,7 +309,7 @@ extract_header_value (char *header, char *dest, char *match)
 }
 
 error_code
-httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
+http_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
 {
     int rc;
     char *start;
@@ -285,7 +324,7 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
 
     /* Parse the url here, before doing memset, because in the 
        recursive case, it is the location referrer */
-    rc = httplib_parse_url (url, &url_info);
+    rc = http_parse_url (url, &url_info);
     if (rc != SR_SUCCESS) return rc;
 
     memset(info, 0, sizeof(SR_HTTP_HEADER));
@@ -296,7 +335,9 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
     start = (char *)strstr(header, "ICY ");
     if (!start) {
 	start = (char *)strstr(header, "HTTP/1.");
-	if (!start) return SR_ERROR_NO_RESPOSE_HEADER;
+	if (!start) {
+	    return SR_ERROR_NO_RESPOSE_HEADER;
+	}
     }
     start = strstr(start, " ") + 1;
     sscanf(start, "%i", &info->icy_code);
@@ -321,24 +362,32 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
     }
 
     // read generic headers
-    extract_header_value(header, info->http_location, "Location:");
-    extract_header_value(header, info->server, "Server:");
-    rc = extract_header_value(header, info->icy_name, "icy-name:");
+    extract_header_value(header, info->http_location, "Location:", 
+			 sizeof(info->http_location));
+    extract_header_value(header, info->server, "Server:", 
+			 sizeof(info->server));
+    rc = extract_header_value(header, info->icy_name, "icy-name:", 
+			      sizeof(info->icy_name));
     if (rc == 0) {
 	/* Icecast 2.0.1 */
-	rc = extract_header_value(header, info->icy_name, "ice-name:");
+	rc = extract_header_value(header, info->icy_name, "ice-name:", 
+				  sizeof(info->icy_name));
     }
     info->have_icy_name = rc;
-    extract_header_value(header, info->icy_url, "icy-url:");
-    rc = extract_header_value(header, stempbr, "icy-br:");
+    extract_header_value(header, info->icy_url, "icy-url:", 
+			 sizeof(info->icy_url));
+    rc = extract_header_value(header, stempbr, 
+			      "icy-br:", sizeof(stempbr));
     if (rc) {
 	info->icy_bitrate = atoi(stempbr);
     }
 
     /* interpret the content type from http header */
-    rc = extract_header_value(header, stempbr, "Content-Type:");
+    rc = extract_header_value(header, stempbr, 
+			      "Content-Type:", sizeof(stempbr));
     if (rc == 0) {
-        rc = extract_header_value(header, stempbr, "content-type:");
+        rc = extract_header_value(header, stempbr, 
+				  "content-type:", sizeof(stempbr));
     }
     if (rc == 0) {
 	info->content_type = CONTENT_TYPE_UNKNOWN;
@@ -361,6 +410,9 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
     else if (strstr(stempbr,"audio/x-scpls")) {
 	info->content_type = CONTENT_TYPE_PLS;
     }
+    else if (strstr(stempbr,"text/html")) {
+	return SR_ERROR_NO_RESPOSE_HEADER;
+    }
     else {
 	info->content_type = CONTENT_TYPE_UNKNOWN;
     }
@@ -418,15 +470,39 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
 	}
 
 	// icecast 1.x headers.
-	extract_header_value(header, info->icy_url, "x-audiocast-server-url:");
-	rc = extract_header_value(header, info->icy_name, "x-audiocast-name:");
+	extract_header_value(header, info->icy_url, "x-audiocast-server-url:",
+			     sizeof(info->icy_url));
+	rc = extract_header_value(header, info->icy_name, "x-audiocast-name:",
+				  sizeof(info->icy_name));
 	info->have_icy_name |= rc;
-	extract_header_value(header, info->icy_genre, "x-audiocast-genre:");
-	rc = extract_header_value(header, stempbr, "x-audiocast-bitrate:");
+	extract_header_value(header, info->icy_genre, "x-audiocast-genre:",
+			     sizeof(info->icy_genre));
+	rc = extract_header_value(header, stempbr, "x-audiocast-bitrate:",
+				  sizeof(stempbr));
 	if (rc) {
 	    info->icy_bitrate = atoi(stempbr);
 	}
     }
+
+
+    // Check for Apache
+    else if (((start = (char *)strstr(header, "Apache")) != NULL) &&
+	     ((start = (char *)strstr(header, "x-audiocast-name")) != NULL)) {
+	// Streaming straight from apache with audiocast headers
+	extract_header_value(header, info->icy_url, "x-audiocast-server-url:",
+			     sizeof(info->icy_url));
+	rc = extract_header_value(header, info->icy_name, "x-audiocast-name:",
+				  sizeof(info->icy_name));
+	info->have_icy_name |= rc;
+	extract_header_value(header, info->icy_genre, "x-audiocast-genre:",
+			     sizeof(info->icy_genre));
+	rc = extract_header_value(header, stempbr, "x-audiocast-bitrate:",
+				  sizeof(stempbr));
+	if (rc) {
+	    info->icy_bitrate = atoi(stempbr);
+	}
+    }
+
     // WTF is Zwitterion?
     else if ((start = (char *)strstr(header, "Zwitterion v")) != NULL) {
 	sscanf(start, "%[^<]<", info->server);
@@ -473,7 +549,7 @@ httplib_parse_sc_header (const char *url, char *header, SR_HTTP_HEADER *info)
  * added to the header
  */
 error_code
-httplib_construct_sc_response(SR_HTTP_HEADER *info, char *header, int size, int icy_meta_support)
+http_construct_sc_response(SR_HTTP_HEADER *info, char *header, int size, int icy_meta_support)
 {
     char *buf = (char *)malloc(size);
 
@@ -596,8 +672,8 @@ Title1=(#1 - 530/18385) GCS hit radio
 Length1=-1
 Version=2
 */
-error_code
-httplib_get_pls (HSOCKET *sock, SR_HTTP_HEADER *info)
+static error_code
+http_get_pls (RIP_MANAGER_INFO* rmi, HSOCKET *sock, SR_HTTP_HEADER *info)
 {
     int s, bytes;
     error_code rc;
@@ -607,7 +683,7 @@ httplib_get_pls (HSOCKET *sock, SR_HTTP_HEADER *info)
     const int timeout = 30;
 
     debug_printf ("Reading pls\n");
-    bytes = socklib_recvall (sock, buf, MAX_PLS_LEN, timeout);
+    bytes = socklib_recvall (rmi, sock, buf, MAX_PLS_LEN, timeout);
     if (bytes < SR_SUCCESS) return bytes;
     if (bytes == 0 || bytes == MAX_PLS_LEN) {
 	debug_printf("Failed in getting PLS (%d bytes)\n", bytes);
@@ -626,7 +702,8 @@ httplib_get_pls (HSOCKET *sock, SR_HTTP_HEADER *info)
 	int best_open = 0;
 
 	sprintf (buf1, "File%d=", s);
-	if (!extract_header_value (buf, location_buf, buf1)) {
+	if (!extract_header_value (buf, location_buf, buf1, 
+				   sizeof(location_buf))) {
 	    break;
 	}
 	if (s == 1) {
@@ -635,7 +712,7 @@ httplib_get_pls (HSOCKET *sock, SR_HTTP_HEADER *info)
 	}
 	
 	sprintf (buf1, "Title%d=", s);
-	if (!extract_header_value (buf, title_buf, buf1)) {
+	if (!extract_header_value (buf, title_buf, buf1, sizeof(title_buf))) {
 	    break;
 	}
 	num_scanned = sscanf (title_buf, "(#%*[0-9] - %d/%d",&used,&total);
@@ -659,8 +736,8 @@ httplib_get_pls (HSOCKET *sock, SR_HTTP_HEADER *info)
 #EXTINF:111,3rd Bass - Al z A-B-Cee z
 mp3/3rd Bass/3rd bass - Al z A-B-Cee z.mp3
 */
-error_code
-httplib_get_m3u (HSOCKET *sock, SR_HTTP_HEADER *info)
+static error_code
+http_get_m3u (RIP_MANAGER_INFO* rmi, HSOCKET *sock, SR_HTTP_HEADER *info)
 {
     int bytes;
     char buf[MAX_M3U_LEN];
@@ -668,7 +745,7 @@ httplib_get_m3u (HSOCKET *sock, SR_HTTP_HEADER *info)
     char* p;
 
     debug_printf ("Reading m3u\n");
-    bytes = socklib_recvall (sock, buf, MAX_M3U_LEN, timeout);
+    bytes = socklib_recvall (rmi, sock, buf, MAX_M3U_LEN, timeout);
     if (bytes < SR_SUCCESS) return bytes;
     if (bytes == 0 || bytes == MAX_M3U_LEN) {
 	debug_printf("Failed in getting M3U (%d bytes)\n", bytes);
@@ -699,22 +776,24 @@ httplib_get_m3u (HSOCKET *sock, SR_HTTP_HEADER *info)
 }
 
 static error_code
-httplib_get_sc_header(const char* url, HSOCKET *sock, SR_HTTP_HEADER *info)
+http_get_sc_header(RIP_MANAGER_INFO* rmi, const char* url, 
+		   HSOCKET *sock, SR_HTTP_HEADER *info)
 {
     int ret;
     char headbuf[MAX_HEADER_LEN] = {'\0'};
 
-    if ((ret = socklib_read_header(sock, headbuf, MAX_HEADER_LEN, NULL)) != SR_SUCCESS)
+    ret = socklib_read_header (rmi, sock, headbuf, MAX_HEADER_LEN);
+    if (ret != SR_SUCCESS)
 	return ret;
 
-    if ((ret = httplib_parse_sc_header(url, headbuf, info)) != SR_SUCCESS)
+    if ((ret = http_parse_sc_header(url, headbuf, info)) != SR_SUCCESS)
 	return ret;
 
     if (info->content_type == CONTENT_TYPE_PLS) {
-	ret = httplib_get_pls (sock, info);
+	ret = http_get_pls (rmi, sock, info);
 	if (ret != SR_SUCCESS) return ret;
     } else if (info->content_type == CONTENT_TYPE_M3U) {
-	ret = httplib_get_m3u (sock, info);
+	ret = http_get_m3u (rmi, sock, info);
 	if (ret != SR_SUCCESS) return ret;
     }
 
