@@ -30,6 +30,7 @@
 #if defined HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <ctype.h>
 #if defined HAVE_WCHAR_SUPPORT
 #if defined HAVE_WCHAR_H
 #include <wchar.h>
@@ -69,18 +70,9 @@
  *****************************************************************************/
 char	*left_str(char *str, int len);
 char	*subnstr_until(const char *str, char *until, char *newstr, int maxlen);
-// char	*strip_invalid_chars(char *str);
 char	*format_byte_size(char *str, long size);
 void	trim(char *str);
 
-/*****************************************************************************
- * Private global variables
- *****************************************************************************/
-const char* m_codeset_locale;
-const char* m_codeset_filesys;
-const char* m_codeset_id3;
-const char* m_codeset_metadata;
-const char* m_codeset_relay;
 
 /*****************************************************************************
  * These functions are NOT mchar related
@@ -129,19 +121,44 @@ char *format_byte_size(char *str, long size)
 
 void trim(char *str)
 {
-    int size = strlen(str)-1;
-    while(str[size] == 13 || str[size] == 10 || str[size] == ' ')
-    {
-	str[size] = '\0';
-	size--;
+    char *start = str;
+    char *end;
+    char *original_end;
+    char *test;
+    
+    /* skip over initial whitespace */
+    while (*start && isspace(*start)) {
+        ++start;
     }
-    size = strlen(str);
-    while(str[0] == ' ')
-    {
-	size--;
-	memmove(str, str+1, size);
+    
+    /* locate end of string */
+    end = start;
+    while (*end) {
+        ++end;
+    }
+    original_end = end;
+    
+    /* backtrack over final whitespace */
+    while (end > start) {
+        test = end-1;
+        if (isspace(*test)) {
+            end = test;
+        } else {
+            break;
+        }
+    }
+    
+    /* move non-whitespace text if initial whitespace was found above. */
+    /* move is unnecessary if resulting string is empty */
+    if (start > str && start != end) {
+        memmove(str, start, end-start);
+    }
+    
+    /* null-terminate resulting string. */
+    /* this is necessary in all cases except when the string was not modified */
+    if (start > str || end < original_end) {
+        str[end-start] = '\0';
     }
-    str[size] = '\0';
 }
 
 /* This is a little different from standard strncpy, because:
@@ -285,34 +302,123 @@ wchar_from_char (char c, const char* codeset)
 }
 #endif /* HAVE_WCHAR_SUPPORT */
 
+/* From advice of Behdad Esfahbod" <behdad@behdad.org> 
+Very good point.  I raised this here too:
+  http://bugzilla.gnome.org/show_bug.cgi?id=391261#c9
+In Pango these days I loop over the string, calling g_utf8_validate()
+and replacing any invalid bytes with -1.  The -1 byte is known to be
+safe when passed to various glib UTF-8 functions.
+*/
+#if defined (commentout)
+int
+utf8validate (gchar* s)
+{
+}
+#endif
+
+/* Assumes src is valid utf8 */
+int
+utf8cpy (gchar* dst, gchar* src, int dst_len)
+{
+    gchar *s = src;
+    gchar *d = dst;
+    gunichar c;
+    gint dlen = 0;
+    gint clen;
+
+    while (dst_len > 6) {
+	c = g_utf8_get_char(s);
+	if (!c) break;
+	clen = g_unichar_to_utf8 (c, d);
+	d += clen;
+	dlen += clen;
+	dst_len -= clen;
+	s = g_utf8_next_char (s);
+    }
+    *d = 0;
+    return dlen;
+}
+
 /* Input value mlen is measured in mchar, not bytes.
    Return value is the number of mchar occupied by the converted string, 
-   not including the null character. */
+   not including the null character. 
+   
+   For GLIB UTF8, it makes more sense to return the dynamically allocated 
+   string, and pass in the codeset string itself rather than 
+   rmi & codeset type. 
+   
+   GLIB UTF8 returns number of bytes.
+*/
 int
-mstring_from_string (mchar* m, int mlen, char* c, int codeset_type)
+mstring_from_string (RIP_MANAGER_INFO* rmi, mchar* m, int mlen, 
+		     char* c, int codeset_type)
 {
+    CODESET_OPTIONS* mchar_cs = &rmi->mchar_cs;
     if (mlen < 0) return 0;
     *m = 0;
     if (!c) return 0;
-#if defined (HAVE_WCHAR_SUPPORT)
+
+#if USE_GLIB_UTF8
+    {
+	GError *error = NULL;
+	gchar* mstring;
+	char* src_codeset;
+	gsize tmp;
+	int rc;
+	
+	switch (codeset_type) {
+	case CODESET_UTF8:
+	    src_codeset = "UTF-8";
+	    break;
+	case CODESET_LOCALE:
+	    src_codeset = mchar_cs->codeset_locale;
+	    break;
+	case CODESET_FILESYS:
+	    src_codeset = mchar_cs->codeset_filesys;
+	    break;
+	case CODESET_ID3:
+	    src_codeset = mchar_cs->codeset_id3;
+	    break;
+	case CODESET_METADATA:
+	    src_codeset = mchar_cs->codeset_metadata;
+	    break;
+	case CODESET_RELAY:
+	    src_codeset = mchar_cs->codeset_relay;
+	    break;
+	default:
+	    printf ("Program error.  Bad codeset m->c (%d)\n", codeset_type);
+	    exit (-1);
+	}
+	mstring = g_convert_with_fallback 
+		(c, -1, "UTF-8", src_codeset, "?", 0, &tmp, &error);
+	if (error) {
+	    debug_printf ("Error converting mstring_from_string\n");
+	    g_free (mstring);
+	    return 0;
+	}
+	rc = utf8cpy (m, mstring, mlen);
+	g_free (mstring);
+	return rc;
+    }
+#elif HAVE_WCHAR_SUPPORT
     switch (codeset_type) {
     case CODESET_UTF8:
 	return wstring_from_string (m, mlen, c, "UTF-8");
 	break;
     case CODESET_LOCALE:
-	return wstring_from_string (m, mlen, c, m_codeset_locale);
+	return wstring_from_string (m, mlen, c, mchar_cs->codeset_locale);
 	break;
     case CODESET_FILESYS:
-	return wstring_from_string (m, mlen, c, m_codeset_filesys);
+	return wstring_from_string (m, mlen, c, mchar_cs->codeset_filesys);
 	break;
     case CODESET_ID3:
-	return wstring_from_string (m, mlen, c, m_codeset_id3);
+	return wstring_from_string (m, mlen, c, mchar_cs->codeset_id3);
 	break;
     case CODESET_METADATA:
-	return wstring_from_string (m, mlen, c, m_codeset_metadata);
+	return wstring_from_string (m, mlen, c, mchar_cs->codeset_metadata);
 	break;
     case CODESET_RELAY:
-	return wstring_from_string (m, mlen, c, m_codeset_relay);
+	return wstring_from_string (m, mlen, c, mchar_cs->codeset_relay);
 	break;
     default:
 	printf ("Program error.  Bad codeset m->c (%d)\n", codeset_type);
@@ -324,33 +430,95 @@ mstring_from_string (mchar* m, int mlen, char* c, int codeset_type)
 #endif
 }
 
+/* Replacement for mstring_from_string using dynamic allocation */
+gchar*
+utf8_string_from_string (char* src, char* codeset)
+{
+    GError *error = NULL;
+    gchar* utf8_string;
+    gsize tmp;
+	
+    utf8_string = g_convert_with_fallback 
+	    (src, -1, "UTF-8", codeset, "?", 0, &tmp, &error);
+    if (error) {
+	debug_printf ("Error converting utf8_string_from_string\n");
+	/* If there's an error, return partial string */
+    }
+    return utf8_string;
+}
+
 /* Return value is the number of char occupied by the converted string, 
    not including the null character. */
 int
-string_from_mstring (char* c, int clen, mchar* m, int codeset_type)
+string_from_mstring (RIP_MANAGER_INFO* rmi, char* c, int clen, mchar* m, int codeset_type)
 {
+    CODESET_OPTIONS* mchar_cs = &rmi->mchar_cs;
     if (clen < 0) return 0;
     *c = 0;
     if (!m) return 0;
-#if defined (HAVE_WCHAR_SUPPORT)
+#if USE_GLIB_UTF8
+    {
+	GError *error = NULL;
+	gchar* cstring;
+	char* tgt_codeset;
+	gsize tmp;
+	int rc;
+	
+	switch (codeset_type) {
+	case CODESET_UTF8:
+	    tgt_codeset = "UTF-8";
+	    break;
+	case CODESET_LOCALE:
+	    tgt_codeset = mchar_cs->codeset_locale;
+	    break;
+	case CODESET_FILESYS:
+	    tgt_codeset = mchar_cs->codeset_filesys;
+	    break;
+	case CODESET_ID3:
+	    tgt_codeset = mchar_cs->codeset_id3;
+	    break;
+	case CODESET_METADATA:
+	    tgt_codeset = mchar_cs->codeset_metadata;
+	    break;
+	case CODESET_RELAY:
+	    tgt_codeset = mchar_cs->codeset_relay;
+	    break;
+	default:
+	    printf ("Program error.  Bad codeset m->c (%d)\n", codeset_type);
+	    exit (-1);
+	}
+	cstring = g_convert_with_fallback 
+		(m, -1, tgt_codeset, "UTF-8", "?", 0, &tmp, &error);
+	if (error) {
+	    debug_printf ("Error converting string_from_mstring\n");
+	    g_free (cstring);
+	    return 0;
+	}
+	/* GCS FIX: truncation can chop multibyte string */
+	/* This will be fixed by using dynamic memory here... */
+	rc = g_strlcpy (c, cstring, clen);
+	g_free (cstring);
+	return strlen(c);
+    }
+#elif defined (HAVE_WCHAR_SUPPORT)
     switch (codeset_type) {
     case CODESET_UTF8:
 	return string_from_wstring (c, clen, m, "UTF-8");
 	break;
     case CODESET_LOCALE:
-	return string_from_wstring (c, clen, m, m_codeset_locale);
+	return string_from_wstring (c, clen, m, mchar_cs->codeset_locale);
 	break;
     case CODESET_FILESYS:
-	return string_from_wstring (c, clen, m, m_codeset_filesys);
+	return string_from_wstring (c, clen, m, mchar_cs->codeset_filesys);
 	break;
     case CODESET_ID3:
-	return string_from_wstring (c, clen, m, m_codeset_id3);
+	return string_from_wstring (c, clen, m, mchar_cs->codeset_id3);
 	break;
     case CODESET_METADATA:
-	return string_from_wstring (c, clen, m, m_codeset_metadata);
+	return string_from_wstring (c, clen, m, mchar_cs->codeset_metadata);
 	break;
     case CODESET_RELAY:
-	return string_from_wstring (c, clen, m, m_codeset_relay);
+	return string_from_wstring (c, clen, m, mchar_cs->codeset_relay);
 	break;
     default:
 	printf ("Program error.  Bad codeset c->m.\n");
@@ -362,38 +530,6 @@ string_from_mstring (char* c, int clen, mchar* m, int codeset_type)
 #endif
 }
 
-mchar
-mchar_from_char (char c, int codeset_type)
-{
-#if defined (HAVE_WCHAR_SUPPORT)
-    switch (codeset_type) {
-    case CODESET_UTF8:
-	return wchar_from_char (c, "UTF-8");
-	break;
-    case CODESET_LOCALE:
-	return wchar_from_char (c, m_codeset_locale);
-	break;
-    case CODESET_FILESYS:
-	return wchar_from_char (c, m_codeset_filesys);
-	break;
-    case CODESET_ID3:
-	return wchar_from_char (c, m_codeset_id3);
-	break;
-    case CODESET_METADATA:
-	return wchar_from_char (c, m_codeset_metadata);
-	break;
-    case CODESET_RELAY:
-	return wchar_from_char (c, m_codeset_relay);
-	break;
-    default:
-	printf ("Program error.  Bad codeset c->m.\n");
-	exit (-1);
-    }
-#else
-    return c;
-#endif
-}
-
 const char*
 default_codeset (void)
 {
@@ -455,13 +591,13 @@ set_codesets_default (CODESET_OPTIONS* cs_opt)
 
     /* I could potentially add stuff like forcing filesys to be utf8 
        (or whatever) for osx here */
-    
 }
 
-/* This sets the global variables (ugh) */
 void
-register_codesets (CODESET_OPTIONS* cs_opt)
+register_codesets (RIP_MANAGER_INFO* rmi, CODESET_OPTIONS* cs_opt)
 {
+    CODESET_OPTIONS* mchar_cs = &rmi->mchar_cs;
+
     /* For ID3, force UCS-2, UCS-2LE, UCS-2BE, UTF-16LE, and UTF-16BE 
        to be UTF-16.  This way, we get the BOM like we need.
        This might change if we upgrade to id3v2.4, which allows 
@@ -471,26 +607,25 @@ register_codesets (CODESET_OPTIONS* cs_opt)
 	strcpy (cs_opt->codeset_id3, "UTF-16");
     }
 
-    m_codeset_locale = cs_opt->codeset_locale;
-    m_codeset_filesys = cs_opt->codeset_filesys;
-    m_codeset_id3 = cs_opt->codeset_id3;
-    m_codeset_metadata = cs_opt->codeset_metadata;
-    m_codeset_relay = cs_opt->codeset_relay;
-    debug_printf ("Locale codeset: %s\n", m_codeset_locale);
-    debug_printf ("Filesys codeset: %s\n", m_codeset_filesys);
-    debug_printf ("ID3 codeset: %s\n", m_codeset_id3);
-    debug_printf ("Metadata codeset: %s\n", m_codeset_metadata);
-    debug_printf ("Relay codeset: %s\n", m_codeset_relay);
-
-
+    strcpy (mchar_cs->codeset_locale, cs_opt->codeset_locale);
+    strcpy (mchar_cs->codeset_filesys, cs_opt->codeset_filesys);
+    strcpy (mchar_cs->codeset_id3, cs_opt->codeset_id3);
+    strcpy (mchar_cs->codeset_metadata, cs_opt->codeset_metadata);
+    strcpy (mchar_cs->codeset_relay, cs_opt->codeset_relay);
+    debug_printf ("Locale codeset: %s\n", mchar_cs->codeset_locale);
+    debug_printf ("Filesys codeset: %s\n", mchar_cs->codeset_filesys);
+    debug_printf ("ID3 codeset: %s\n", mchar_cs->codeset_id3);
+    debug_printf ("Metadata codeset: %s\n", mchar_cs->codeset_metadata);
+    debug_printf ("Relay codeset: %s\n", mchar_cs->codeset_relay);
 }
 
 /* This is used to set the codeset byte for id3v2 frames */
 int
-is_id3_unicode (void)
+is_id3_unicode (RIP_MANAGER_INFO* rmi)
 {
+    CODESET_OPTIONS* mchar_cs = &rmi->mchar_cs;
 #if HAVE_WCHAR_SUPPORT
-    if (!strcmp ("UTF-16", m_codeset_id3)) {
+    if (!strcmp ("UTF-16", mchar_cs->codeset_id3)) {
 	return 1;
     }
 #endif
@@ -512,7 +647,9 @@ mstrncpy (mchar* dst, mchar* src, int n)
 mchar* 
 mstrdup (mchar* src)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    return strdup (src);
+#elif defined HAVE_WCHAR_SUPPORT
     /* wstrdup/wcsdup is non-standard */
     mchar* new_string = (mchar*) malloc (sizeof(mchar)*(wcslen(src) + 1));
     wcscpy (new_string, src);
@@ -525,7 +662,9 @@ mstrdup (mchar* src)
 mchar* 
 mstrcpy (mchar* dest, const mchar* src)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    return strcpy (dest, src);
+#elif defined HAVE_WCHAR_SUPPORT
     return wcscpy (dest, src);
 #else
     return strcpy (dest, src);
@@ -535,7 +674,9 @@ mstrcpy (mchar* dest, const mchar* src)
 size_t
 mstrlen (mchar* s)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    return strlen (s);
+#elif defined HAVE_WCHAR_SUPPORT
     return wcslen (s);
 #else
     return strlen (s);
@@ -550,7 +691,9 @@ msnprintf (mchar* dest, size_t n, const mchar* fmt, ...)
     int rc;
     va_list ap;
     va_start (ap, fmt);
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    rc = vsnprintf (dest, n, fmt, ap);
+#elif defined HAVE_WCHAR_SUPPORT
     rc = vswprintf (dest, n, fmt, ap);
     debug_printf ("vswprintf got %d\n", rc);
 #else
@@ -563,17 +706,33 @@ msnprintf (mchar* dest, size_t n, const mchar* fmt, ...)
 mchar*
 mstrchr (const mchar* ws, mchar wc)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    return g_utf8_strchr (ws, -1, g_utf8_get_char(&wc));
+#elif defined HAVE_WCHAR_SUPPORT
     return wcschr (ws, wc);
 #else
     return strchr (ws, wc);
 #endif
 }
 
+mchar*
+mstrrchr (const mchar* ws, mchar wc)
+{
+#if USE_GLIB_UTF8
+    return g_utf8_strrchr (ws, -1, g_utf8_get_char(&wc));
+#elif defined HAVE_WCHAR_SUPPORT
+    return wcsrchr (ws, wc);
+#else
+    return strrchr (ws, wc);
+#endif
+}
+
 mchar*
 mstrncat (mchar* ws1, const mchar* ws2, size_t n)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    return strncat (ws1, ws2, n);
+#elif defined HAVE_WCHAR_SUPPORT
     return wcsncat (ws1, ws2, n);
 #else
     return strncat (ws1, ws2, n);
@@ -583,7 +742,9 @@ mstrncat (mchar* ws1, const mchar* ws2, size_t n)
 int
 mstrcmp (const mchar* ws1, const mchar* ws2)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    return strcmp (ws1, ws2);
+#elif defined HAVE_WCHAR_SUPPORT
     return wcscmp (ws1, ws2);
 #else
     return strcmp (ws1, ws2);
@@ -593,7 +754,9 @@ mstrcmp (const mchar* ws1, const mchar* ws2)
 long int
 mtol (const mchar* string)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    return strtol (string, 0, 0);
+#elif defined HAVE_WCHAR_SUPPORT
     return wcstol (string, 0, 0);
 #else
     return strtol (string, 0, 0);
