@@ -23,7 +23,6 @@
 #include "mchar.h"
 #include "debug.h"
 #include "srtypes.h"
-#include "regex.h"
 
 /* GCS FIX: figure out function prototypes for regwcomp, regwexec */
 
@@ -43,25 +42,15 @@
 #define PARSERULE_EXCLUDE            0x10
 
 
-struct parse_rule {
-    int cmd;
-    int flags;
-    int artist_idx;
-    int title_idx;
-    int album_idx;
-    int trackno_idx;
-    regex_t* reg;
-    mchar* match;
-    mchar* subst;
-};
-typedef struct parse_rule Parse_Rule;
-
 static Parse_Rule m_default_rule_list[] = {
     { PARSERULE_CMD_MATCH, 
       PARSERULE_SKIP,
-      0, 0, 0, 0,
+      0, 0, 0, 0, 0,
       0, 
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+      "^A suivre:",
+      ""
+#elif defined HAVE_WCHAR_SUPPORT
       /* GCS FIX: L string literals are not good */
       L"^A suivre:",
       L""
@@ -72,9 +61,12 @@ static Parse_Rule m_default_rule_list[] = {
     },
     { PARSERULE_CMD_SUBST, 
       PARSERULE_ICASE,
-      0, 0, 0, 0,
-      0, 
-#if defined HAVE_WCHAR_SUPPORT
+      0, 0, 0, 0, 0,
+      0,
+#if USE_GLIB_UTF8
+      "[[:space:]]*-?[[:space:]]*mp3pro$",
+      ""
+#elif defined HAVE_WCHAR_SUPPORT
       L"[[:space:]]*-?[[:space:]]*mp3pro$",
       L""
 #else
@@ -84,9 +76,12 @@ static Parse_Rule m_default_rule_list[] = {
     },
     { PARSERULE_CMD_MATCH, 
       PARSERULE_ICASE,
-      1, 2, 0, 0,
-      0, 
-#if defined HAVE_WCHAR_SUPPORT
+      1, 2, 0, 0, 0,
+      0,
+#if USE_GLIB_UTF8
+      "^[[:space:]]*([^-]*[^-[:space:]])[[:space:]]*-[[:space:]]*(.*)[[:space:]]*$",
+      ""
+#elif defined HAVE_WCHAR_SUPPORT
       L"^[[:space:]]*([^-]*[^-[:space:]])[[:space:]]*-[[:space:]]*(.*)[[:space:]]*$",
       L""
 #else
@@ -96,9 +91,12 @@ static Parse_Rule m_default_rule_list[] = {
     },
     { 0x00, 
       0x00,
-      0, 0, 0, 0,
-      0, 
-#if defined HAVE_WCHAR_SUPPORT
+      0, 0, 0, 0, 0,
+      0,
+#if USE_GLIB_UTF8
+      "",
+      ""
+#elif defined HAVE_WCHAR_SUPPORT
       L"",
       L""
 #else
@@ -107,7 +105,8 @@ static Parse_Rule m_default_rule_list[] = {
 #endif
     }
 };
-static Parse_Rule* m_global_rule_list = m_default_rule_list;
+
+// static Parse_Rule* m_global_rule_list = m_default_rule_list;
 
 /*****************************************************************************
  * Public functions
@@ -115,7 +114,10 @@ static Parse_Rule* m_global_rule_list = m_default_rule_list;
 static int
 sr_regcomp (Parse_Rule* pr, mchar* rule_string, int cflags)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    /* GCS FIX: Use PCRE for utf8 */
+    return regcomp(pr->reg, rule_string, cflags);
+#elif defined HAVE_WCHAR_SUPPORT
     return regwcomp(pr->reg, rule_string, cflags);
 #else
     return regcomp(pr->reg, rule_string, cflags);
@@ -126,7 +128,10 @@ static int
 mregexec (const regex_t* preg, const mchar* string, size_t nmatch,
 	  regmatch_t pmatch[], int eflags)
 {
-#if defined HAVE_WCHAR_SUPPORT
+#if USE_GLIB_UTF8
+    /* GCS FIX: Use PCRE for utf8 */
+    return regexec (preg, string, nmatch, pmatch, eflags);
+#elif defined HAVE_WCHAR_SUPPORT
     return regwexec (preg, string, nmatch, pmatch, eflags);
 #else
     return regexec (preg, string, nmatch, pmatch, eflags);
@@ -156,15 +161,16 @@ compile_rule (Parse_Rule* pr, mchar* rule_string)
 }
 
 static void
-use_default_rules (void)
+use_default_rules (RIP_MANAGER_INFO* rmi)
 {
     Parse_Rule* rulep;
 
     /* set global rule list to default */
-    m_global_rule_list = m_default_rule_list;
+    rmi->parse_rules = (Parse_Rule*) malloc (sizeof(m_default_rule_list));
+    memcpy (rmi->parse_rules, m_default_rule_list, sizeof(m_default_rule_list));
 
     /* compile regular expressions */
-    for (rulep = m_global_rule_list; rulep->cmd; rulep++) {
+    for (rulep = rmi->parse_rules; rulep->cmd; rulep++) {
 	compile_rule (rulep, rulep->match);
     }
 }
@@ -219,6 +225,9 @@ parse_flags (Parse_Rule* pr, char* flags)
 	case 'T':
 	    tag = &pr->title_idx;
 	    break;
+	case 'Y':
+	    tag = &pr->year_idx;
+	    break;
 	case 0:
 	case '\n':
 	case '\r':
@@ -272,42 +281,51 @@ parse_escaped_string (char* outbuf, char* inbuf)
 }
 
 /* This mega-function reads in the rules file, and loads 
-    all the rules into the m_global_rules_list data structure */
+    all the rules into the rmi->parse_rules data structure */
 void
-init_metadata_parser (char* rules_file)
+init_metadata_parser (RIP_MANAGER_INFO* rmi, char* rules_file)
 {
     FILE* fp;
     int ri;     /* Rule index */
+    int rn;     /* Number of rules allocated */
 
     if (!rules_file || !*rules_file) {
-	use_default_rules ();
+	use_default_rules (rmi);
 	return;
     }
     fp = fopen (rules_file, "r");
     if (!fp) {
-	use_default_rules ();
+	use_default_rules (rmi);
 	return;
     }
 
-    m_global_rule_list = 0;
-    ri = 0;
-    while (!feof(fp)) {
+    rmi->parse_rules = 0;
+    ri = rn = 0;
+    while (1) {
 	char rule_buf[MAX_RULE_SIZE];
 	char match_buf[MAX_RULE_SIZE];
 	char subst_buf[MAX_RULE_SIZE];
 	mchar w_match_buf[MAX_RULE_SIZE];
 	mchar w_subst_buf[MAX_RULE_SIZE];
 	char* rbp;
+	char* rp;
 	int got_command;
 	int rc;
 	
-	/* Allocate memory for rule */
-	m_global_rule_list = realloc (m_global_rule_list,
-				      (ri+1) * sizeof(Parse_Rule));
-	memset (&m_global_rule_list[ri], 0, sizeof(Parse_Rule));
+	/* Allocate memory for rule, if necessary. */
+	/* If there are no more rules in the file, */
+	/* this rule will become the sentinel null rule */
+	if (ri+1 != rn) {
+	    rmi->parse_rules = realloc (rmi->parse_rules,
+					(ri+1) * sizeof(Parse_Rule));
+	    memset (&rmi->parse_rules[ri], 0, sizeof(Parse_Rule));
+	    rn = ri+1;
+	}
+
 
 	/* Get next line from file */
-	fgets (rule_buf,2048,fp);
+	rp = fgets (rule_buf,2048,fp);
+	if (!rp) break;
 
 	/* Skip leading whitespace */
 	rbp = rule_buf;
@@ -319,11 +337,11 @@ init_metadata_parser (char* rules_file)
 	switch (*rbp++) {
 	case 'm':
 	    got_command = 1;
-	    m_global_rule_list[ri].cmd = PARSERULE_CMD_MATCH;
+	    rmi->parse_rules[ri].cmd = PARSERULE_CMD_MATCH;
 	    break;
 	case 's':
 	    got_command = 1;
-	    m_global_rule_list[ri].cmd = PARSERULE_CMD_SUBST;
+	    rmi->parse_rules[ri].cmd = PARSERULE_CMD_SUBST;
 	    break;
 	case '#':
 	    got_command = 0;
@@ -353,7 +371,7 @@ init_metadata_parser (char* rules_file)
 	}
 
 	/* Parse subst string */
-	if (m_global_rule_list[ri].cmd == PARSERULE_CMD_SUBST) {
+	if (rmi->parse_rules[ri].cmd == PARSERULE_CMD_SUBST) {
 	    rbp = parse_escaped_string (subst_buf, rbp);
 	    debug_printf ("subst_buf=%s\n", subst_buf);
 	    if (!rbp) {
@@ -364,7 +382,7 @@ init_metadata_parser (char* rules_file)
 	}
 
 	/* Parse flags */
-	rc = parse_flags (&m_global_rule_list[ri], rbp);
+	rc = parse_flags (&rmi->parse_rules[ri], rbp);
 	if (!rc) {
 	    printf ("Warning: malformed command in rules file:\n%s\n",
 		    rule_buf);
@@ -373,9 +391,9 @@ init_metadata_parser (char* rules_file)
 
 	/* Compile the rule */
 	debug_printf ("Compiling the rule\n");
-	mstring_from_string (w_match_buf, MAX_RULE_SIZE, match_buf, 
+	mstring_from_string (rmi, w_match_buf, MAX_RULE_SIZE, match_buf, 
 			     CODESET_UTF8);
-	if (!compile_rule(&m_global_rule_list[ri], w_match_buf)) {
+	if (!compile_rule(&rmi->parse_rules[ri], w_match_buf)) {
 	    printf ("Warning: malformed regular expression:\n%s\n", 
 		    match_buf);
 	    continue;
@@ -384,28 +402,25 @@ init_metadata_parser (char* rules_file)
 	/* Copy rule strings */
 	debug_printf ("Copying rule string (1)\n");
 	debug_mprintf (m_("String is ") m_S m_("\n"), w_match_buf);
-	m_global_rule_list[ri].match = mstrdup(w_match_buf);
+	rmi->parse_rules[ri].match = mstrdup(w_match_buf);
 	debug_printf ("Copying rule string (2)\n");
-	if (m_global_rule_list[ri].cmd == PARSERULE_CMD_SUBST) {
+	if (rmi->parse_rules[ri].cmd == PARSERULE_CMD_SUBST) {
 	    debug_printf ("Copying rule string (3)\n");
-	    mstring_from_string (w_subst_buf, MAX_RULE_SIZE, subst_buf, 
+	    mstring_from_string (rmi, w_subst_buf, MAX_RULE_SIZE, subst_buf, 
 				 CODESET_UTF8);
 	    debug_printf ("Copying rule string (4)\n");
-	    m_global_rule_list[ri].subst = mstrdup(w_subst_buf);
+	    rmi->parse_rules[ri].subst = mstrdup(w_subst_buf);
 	    debug_printf ("Copying rule string (5)\n");
 	}
 
 	debug_printf ("End of loop\n");
 	ri++;
     }
-    /* Add null rule to the end */
-    m_global_rule_list = realloc (m_global_rule_list,
-				  (ri+1) * sizeof(Parse_Rule));
-    memset (&m_global_rule_list[ri], 0, sizeof(Parse_Rule));
+    fclose(fp);
 }
 
 void
-compose_metadata (TRACK_INFO* ti)
+compose_metadata (RIP_MANAGER_INFO* rmi, TRACK_INFO* ti)
 {
     int num_bytes;
     unsigned char num_16_bytes;
@@ -425,7 +440,7 @@ compose_metadata (TRACK_INFO* ti)
 	debug_printf ("No track info when composing relay metadata\n");
     }
     debug_printf ("Converting relay string to char\n");
-    num_bytes = string_from_mstring (&ti->composed_metadata[1], 
+    num_bytes = string_from_mstring (rmi, &ti->composed_metadata[1], 
 				     MAX_METADATA_LEN, 
 				     w_composed_metadata, 
 				     CODESET_RELAY);
@@ -435,7 +450,7 @@ compose_metadata (TRACK_INFO* ti)
 }
 
 void
-parse_metadata (TRACK_INFO* ti)
+parse_metadata (RIP_MANAGER_INFO* rmi, TRACK_INFO* ti)
 {
     int eflags;
     int rc;
@@ -463,9 +478,9 @@ parse_metadata (TRACK_INFO* ti)
     /* For now, only default rules supported with ascii 
        regular expressions. */
     debug_printf ("Converting query string to wide\n");
-    mstring_from_string (query_string, MAX_TRACK_LEN, 
+    mstring_from_string (rmi, query_string, MAX_TRACK_LEN, 
 			 ti->raw_metadata, CODESET_METADATA);
-    for (rulep = m_global_rule_list; rulep->cmd; rulep++) {
+    for (rulep = rmi->parse_rules; rulep->cmd; rulep++) {
 	regmatch_t pmatch[MAX_SUBMATCHES+1];
 
 	eflags = 0;
@@ -513,14 +528,18 @@ parse_metadata (TRACK_INFO* ti)
 		copy_rule_result (ti->artist, query_string, pmatch, rulep->artist_idx);
 		copy_rule_result (ti->title, query_string, pmatch, rulep->title_idx);
 		copy_rule_result (ti->album, query_string, pmatch, rulep->album_idx);
+		copy_rule_result (ti->track_p, query_string, pmatch, rulep->trackno_idx);
+		copy_rule_result (ti->year, query_string, pmatch, rulep->year_idx);
 		ti->have_track_info = 1;
-		compose_metadata (ti);
-		/* GCS FIX: We don't have a track number. */
+		compose_metadata (rmi, ti);
 		debug_mprintf (m_("Parsed track info.\n")
 			       m_("ARTIST: ") m_S m_("\n")
 			       m_("TITLE: ")  m_S m_("\n")
-			       m_("ALBUM: ")  m_S m_("\n"),
-			       ti->artist, ti->title, ti->album);
+			       m_("ALBUM: ")  m_S m_("\n")
+			       m_("TRACK: ")  m_S m_("\n")
+			       m_("YEAR: ")  m_S m_("\n"),
+			       ti->artist, ti->title, ti->album,
+                               ti->track_p, ti->year);
 		return;
 	    }
 	}
@@ -555,5 +574,13 @@ parse_metadata (TRACK_INFO* ti)
     debug_printf ("Fell through while parsing data...\n");
     mstrncpy (ti->title, query_string, MAX_TRACK_LEN);
     ti->have_track_info = 1;
-    compose_metadata (ti);
+    compose_metadata (rmi, ti);
 }
+
+void
+parser_free (RIP_MANAGER_INFO* rmi)
+{
+    free (rmi->parse_rules);
+    rmi->parse_rules = 0;
+}
+
